@@ -215,7 +215,7 @@ alpha.scale <- function(n.cut, col="blue") {
 #' @inheritParams disp.obs
 #' @noRd
 overlay.split <- function(g, network, E0=unique(g$data$`50%`[g$data$dose==0]), method="common",
-                          likelihood="binomial", link="logit", ...) {
+                          likelihood="binomial", link="logit", lim="cred", ...) {
 
   # Check/assign link and likelihood
   likelink <- check.likelink(network$data.ab, likelihood=likelihood, link=link)
@@ -245,6 +245,13 @@ overlay.split <- function(g, network, E0=unique(g$data$`50%`[g$data$dose==0]), m
   trtvec <- trtvec[agentvec %in% g$data$agent]
   agentvec <- agentvec[agentvec %in% g$data$agent]
 
+  # Add between-study SD
+  if (lim=="pred" & method=="random") {
+    mat <- array(dim = c(nrow(splitmat), ncol(splitmat), 2))
+    mat[,,1] <- splitmat
+    mat[,,2] <- stats::median(splitNMA$jagsresult$BUGSoutput$sims.list$sd)
+    splitmat <- apply(mat, MARGIN=c(1,2), FUN=function(x) stats::rnorm(1, x[1], x[2]))
+  }
 
   # Ensure E0 is correct length to add to splitmat
   if (length(E0)<nrow(splitmat)) {
@@ -383,7 +390,13 @@ devplot <- function(mbnma, plot.type="box", facet=TRUE, dev.type="resdev",
   }
 
   # Add studyID in addition to study index from model and sort data frame
-  dev.df$studyID <- mbnma$model.arg$jagsdata$studyID
+  studvec <- vector()
+  for (i in seq_along(mbnma$model.arg$jagsdata$studyID)) {
+    studvec <- append(studvec, rep(mbnma$model.arg$jagsdata$studyID[i], mbnma$model.arg$jagsdata$narm[i]))
+  }
+
+  dev.df$studyID <- studvec
+  # dev.df$studyID <- mbnma$model.arg$jagsdata$studyID
   dev.df <- dplyr::arrange(dev.df, dev.df$study, dev.df$arm)
 
   # Plots the residual deviances
@@ -566,7 +579,13 @@ fitplot <- function(mbnma, disp.obs=TRUE,
   }
 
   # Add studyID in addition to study index from model
-  theta.df$studyID <- mbnma$model.arg$jagsdata$studyID
+  studvec <- vector()
+  for (i in seq_along(mbnma$model.arg$jagsdata$studyID)) {
+    studvec <- append(studvec, rep(mbnma$model.arg$jagsdata$studyID[i], mbnma$model.arg$jagsdata$narm[i]))
+  }
+
+  theta.df$studyID <- studvec
+  # theta.df$studyID <- mbnma$model.arg$jagsdata$studyID
 
 
   # Obtain raw responses to plot over fitted
@@ -663,7 +682,7 @@ plot.invisible <- function(...){
 #' @param x An object of class `"mbnma.rank"` generated by `rank.mbnma()`
 #' @param sucra A logical object to indicate whether Surface Under Cumulative Ranking Curve (SUCRA)
 #' values should be calculated and returned as a data frame. Areas calculated
-#' using \code{\link[rgeos]{readWKT}}.
+#' using trapezoid approach.
 #' @param ... Arguments to be sent to `ggplot::geom_line()`
 #' @inheritParams rank.mbnma
 #'
